#ifndef __CHM_ERROR_CLASS_H__
#define __CHM_ERROR_CLASS_H__
//---------------------------------------------------------------------------
// Copyright (C) 1997-2007 iNTERFACEWARE Inc.  All Rights Reserved
//
// Module: CHMerrorClass
//
// Author: Eliot Muir
//
//---------------------------------------------------------------------------

#include <CHM/CHMminimumInclude.h>
class CHMstring;

const long CHM_BAD_VERSION = 123112;

#ifdef _MFC_VER
#define _USE_MFC_EXCEPTIONS
#endif

#if defined(_USE_MFC_EXCEPTIONS)
#include <afx.h>

class CHMerror : public CException
{
    DECLARE_DYNAMIC(CHMerror)
public:
   CHMerror(const CHMchar* Message, long Line, const char* File, long Code);
   CHMerror(const CHMresult Code, long Line, const char* File);
   CHMerror(const CHMerror& Orig);

   virtual ~CHMerror();

    //void Empty();
    
    virtual BOOL GetErrorMessage(LPTSTR lpszError, UINT nMaxError,
                                PUINT pnHelpContext = NULL);
   const long Code()        const;
   CHMchar*   Description() const;
   const char* Parameter(const CHMchar* pKey) const;

   // This function is deprecated. Use Dump instead. See manual for migration guide.
   // 
   // void printOn(IOostream& Stream) const;
   
   /** Output the current error information to a string.
    * This is useful for diagnostics.
    */  
   CHMstring Dump() const;

//#ifdef _DEBUG
//  void TraceErrorMessage(LPCTSTR szTrace) const;
//#endif // DEBUG
protected:
   CHMchar* m_pDescription;
   long     m_Code;
   long     m_Line;
   char*    m_File;
   CHMreferenceHandle m_Handle;
private:
   CHMerror& operator=(const CHMerror& Error);  // not allowed
};

#else
/** Chameleon exception class.
 * This is the only exception that should be thrown by the Chameleon code.
 * This exception class may inherit from other
 * exception classes depending on the environment.  For MFC for
 * instance, the CHMerror class is defined to inherit from the CException
 * class. For other compilers the CHMerror class is just a stand-alone class.
 */
class CHMerror
{
public:
   CHMerror(const CHMchar* Message, long Line, const char* File, long Code);
   CHMerror(const CHMerror& Error);
   CHMerror(const CHMresult Handle, long Line, const char* File);
   virtual ~CHMerror();

   /** Get the error code. */
   const long      Code()       const;

   /** Get the number of parameters. */
   unsigned int CountOfParameter() const;
   
   /** Get the description of the error. */
   CHMchar* Description() const;

   /** Output the error information to a string.
    * This is useful for diagnostics.
    */
   CHMstring Dump() const;
   
   /** Check whether a parameter with this name is present. */
   CHMboolean HasParameter(const CHMchar* pKey) const; 
   
   /** Get the parameter referenced by the key \c pKey. */
   const char* Parameter(const CHMchar* pKey) const;

   /** Get the name of a parameter by sequence number. */
   const char* ParameterName(unsigned int Index) const;

   // This function is deprecated. Use Dump instead. See manual for migration guide.
   //
   // void printOn(IOostream& Stream) const;
   
 

protected:
   CHMchar*    m_pDescription;
   long        m_Code;
   long        m_Line;
   char* m_File;
   CHMreferenceHandle m_Handle;
private:
   CHMerror& operator=(const CHMerror& Error);  // not allowed
};
#endif  // end of defining compilation of error determined by the type of
        // environment being used...

// This function is deprecated. Use Dump instead. See manual for migration guide.
//
// IOostream& operator<<(IOostream& Stream, const CHMerror& Error);

/** Throw a CHMerror object
 * This is used for implementation of the Chameleon C++ classes.
 */
void CHMactivateCondition(const CHMchar* Message, long Line, const char* File, CHMresult Code);
/** Throw a CHMerror object
 * This is used for implementation of the Chameleon C++ classes.
 */
void CHMactivateCondition(const CHMchar* Message, long Line, const char* File);

#if defined(WIN32) && defined(CHM_64)
#define CHM_CHECK_CALL(x) { CHMresult Result = x; if (Result != CHM_OK) { CHMactivateCondition(CHM_TEXT(#x), __LINE__, __FILE__, Result); } }
#else
#define CHM_CHECK_CALL(x) { CHMresult Result = _##x; if (Result != CHM_OK) { CHMactivateCondition(CHM_TEXT(#x), __LINE__, __FILE__, Result); } }
#endif

#define CHM_ASSERT(x) if (!(x))  { CHMactivateCondition(CHM_TEXT(#x), __LINE__, __FILE__); }

#define CHM_ASSERT_COMMENT(x, y) if (!(x)) { CHMactivateCondition(CHM_TEXT(y), __LINE__, __FILE__);  }

#define CHM_HANDLE_UNKNOWN_MESSAGE throw CHMerror(CHM_TEXT("Unknown message encountered."), __LINE__, __FILE__, 0); 

#ifndef CHMPRECONDITION
#  define CHMPRECONDITION(x) CHM_ASSERT(#x)
#endif

#endif // end of defensive include
