#ifndef __CHM_HL7_CLIENT_SOCKET_H__
#define __CHM_HL7_CLIENT_SOCKET_H__

#include <CHM/CHMminimumInclude.h>

// THIS CLASS IS DEPRECATED.  IT IS ONLY HERE FOR THE PURPOSES OF SOME CUSTOMERS NEEDING TO MIGRATE
// THERE EXISTING CODE.  IT IS NOT SUPPORTED.

class CHMhl7ClientSocket;
struct CHMconnectionEvents
{
   /** Called when a connection closes 
       This function is called after a connection has been closed by the remote host. After this
       event occurs the socket is in a disconnected state and cannot send or receive messages.
       You can reuse the object for another connection using connect() after this event occurs
       To handle this event you'll need to override this function on a base class.
   */
   virtual void onClose( CHMhl7ClientSocket& Client ){}
   
   /** called when a message arrives 
       This function is called when a full message wrapped with the LLP protocol is received
       from a remote host. The message is unwrapped and passes as a parameter to the function. 
       To handle this event you'll need to override this function on a base class.
       \param Client The client object that the message has arrived on. This is useful on the
         CHMhl7ServerSocket class.
       \param Message A null terminated string with the message contents. The LLP wrapper is not
         part of this string.
   */
   virtual void onMessage( CHMhl7ClientSocket& Client, const char* Message ){}

   /** called when an error occurs
       This function is called when an error occurs on the connection. The connection is also closed
       as a result of this error. You can reuse the object to make another connection using the connect()
       function. To handle this event you'll need to override this function on a base class.
       \param Client The client object that the error occurred on. This is usefull on the
         CHMhl7ServerSocket class.
       \param ErrorId The error code. This is a number that identifies the error type.
       \param ErrorMessage A null terminated string with the error message.
   */
   virtual void onTransportError( CHMhl7ClientSocket& Client, int ErrorId, const char* ErrorMessage ){}
};

struct CHMhl7ClientSocketPrivate;

/** Deprecated Client class for asynchronous TCP/IP communication.
 * The CHMhl7ClientSocket represents a TCP/IP connection between two computers. 
 * You can use this class to connect to a server and send/receive messages. The 
 * messages are automatically wrapped and unwrapped using the LLP protocol.
 * To implement a CHMhl7ClientSocket you must create a new class that inheirits 
 * from it. This way you can override the virtual methods to handle the network events.
 *
 * \deprecated
 */
class CHMhl7ClientSocket : public CHMconnectionEvents
{
public:
   /** Constructor
       Constructor used for a client connecting to a server.
       This constructor is used when you would like to create a object to connect
       to a remote host.
       \param Dynamic Specifies whether this object should be 
         automatically deleted when the connection closes. If the object in constructed 
         as a stack variable this should be false. Only set this value to true if you
         create the object with new and don't ever delete it. The default value is false.
   */
   CHMhl7ClientSocket( CHMboolean Dynamic = false );

   /** Constructor
       Constructor used for a server accepting a client's connection.
       This constructor is used when an incoming connection is requested in a 
       server application. The instantiated object represents the server side of the connection.
       \param ClientId Obtained from the OnNewConnection event on CHMhl7ServerSocket. The value
         is the idetifier of a client connection.
       \param Dynamic Specifies whether this object should be 
         automatically deleted when the connection closes. If the object in constructed 
         as a stack variable this should be false. Only set this value to true if you
         create the object with new and don't ever delete it. Typically when the OnNewConnection
         event occurs you dynamically create a new client with this value as true. The default value is 
         true.
   */
   CHMhl7ClientSocket( int ClientId, CHMboolean Dynamic = true );
   
   /** Destructor */
   virtual ~CHMhl7ClientSocket();

   /** Connect to a remote host. 
       Requests a new connection to a remote host on a specified port. The call is asynchronous so
       there is not neccesarily an established connection when the call returns. A connection is
       established when an onConnect() event occurs. If the connection fails you'll receive an
       onTransportError() event. Override these virtual methods on a base class to handle these events.
       \param Host The host name or IP address as a string. This can be an IP address string such as "127.0.0.1"
       a computer name "localhost" or a domain name "interfaceware.com".
       \param Port Specifies the port on the remote host to connect to. 
   */
   void connect( const char* Host, short Port );

   /** Send a message. 
       Sends a message on the connection to the remote host. This call automatically wraps the message
       with the LLP header and footer. The object must be connected to send a message.
       \param Message A null terminated string representing the message that should be sent.
   */
   void sendMessage( const char* Message );

   /** Get the raw IP address 
       \return The 32 bit integer IP address of the remote host.
   */
   CHMuint32 getIpAddress();

   /** Get the IP address as a string.
      \return The IP address as a string in the form "127.0.0.1".
   */
   const char* getIpAddressString();

   /** Gets the host name.
      This function does a lookup for the host name of the remote host's IP address. If no host name is found
      then it returns the IP address of the remote host as a string.
      \return The host name of the remote host.
   */
   const char* getHostName();

   /** Close a connection. 
      Closes the connection to the remote host. If the connection has already been closed this
      function does nothing.
   */
   void closeConnection();

   /** Called when a connection has been established. 
       This function is called after a connection request using the connect() function has been
       successful. After this event occurs the connection is in a state to send and receive messages.
       To handle this event you'll need to override this function on a base class.
   */
   virtual void onConnect(){}

   /** True when the connection is open
   */
   CHMboolean isConnected() const;

   /** Called when a connection closes 
       This function is called after a connection has been closed by the remote host. After this
       event occurs the socket is in a disconnected state and cannot send or receive messages.
       You can reuse the object for another connection using connect() after this event occurs
       To handle this event you'll need to override this function on a base class.
   */
   virtual void onClose( CHMhl7ClientSocket& Client );

   /** the identidy of the client 
      \return The unique id of the connection.
   */
   int id();
private:
   CHMhl7ClientSocketPrivate* pMember;

   // not allowed
   CHMhl7ClientSocket(const CHMhl7ClientSocket& Orig);
   CHMhl7ClientSocket& operator=(const CHMhl7ClientSocket& Orig);
};

#endif // end of defensive include
