#ifndef __CHM_LLP_WRAPPER_H__
#define __CHM_LLP_WRAPPER_H__
//---------------------------------------------------------------------------
// Copyright (C) 1997-2007 iNTERFACEWARE Inc.  All Rights Reserved
//
// Module: CHMllpWrapper
//
// Author: Greg Norman
//
//---------------------------------------------------------------------------
#include <CHM/CHMwrapper.h>

class CHMllpWrapperPrivate;

/** Lower Layer Protocol (LLP) wrapper.
 * This is an implementation of the CHMwrapper object
 * that supports the LLP protocol.
 * The LLP protocol consists of preceding the
 * message with a "\x0B" character and trailing it with "\x1C\x0D"
 *
 * \deprecated
 **/
class CHMllpWrapper
: public CHMwrapper
{
public:
   /** Constructor. */
   CHMllpWrapper();
   /** Destructor. */
   virtual ~CHMllpWrapper();
  
   /** Wrap an outgoing message \c MessageDate. */
   virtual void AddWrapper(CHMstring& MessageData);

   /** Detect and unwrap a message in an incoming stream. 
    * This routine removes the read data from \c Buffer and
    * sets the \c Message string to be equal to the message data
    * and returns true if a message is present in the buffer;
    * otherwise, it just returns false and leaves \c Buffer unaltered
    * and the content of \c Message is undefined.*/
   virtual CHMboolean DetectMessage(CHMstring& Buffer, 
                                    CHMstring& Message);

    /** 
    * Get the LLP header.  By default this is a "\x0B" character. 
    */
   const CHMchar* LLPheader() const;
   
   /** 
    * Get the LLP trailer.  By default this is "\x1C\x0D".
    */
   const CHMchar* LLPtrailer() const;
   
   /** 
    * Set the LLP header.  By default this is a "\x0B" character. 
    */
   void setLLPheader(const CHMchar* NewValue);
  
   /** 
    * Set the LLPtrailer.  By default this is "\x1C\x0D".
    */
   void setLLPtrailer(const CHMchar* NewValue);

private:
   CHMllpWrapper(CHMllpWrapper& Orig);
   CHMllpWrapper& operator=(CHMllpWrapper& Orig);

   CHMllpWrapperPrivate* pMember;
};

#endif // end of defensive include

