#ifndef __CHM_MANUAL_RESET_EVENT_CLASS_H__
#define __CHM_MANUAL_RESET_EVENT_CLASS_H__
//---------------------------------------------------------------------------
// Copyright (C) 1997-2009 iNTERFACEWARE Inc. All Rights Reserved
//
// Module: CHMmanualResetEvent class
//
// Author: Steven Dodd
// Date:   Jan 28, 2009
//
//---------------------------------------------------------------------------
#include <CHM/CHMminimumInclude.h>

/** A portable manual-reset event object, useful for thread synchronization.
 * The event has two states: signalled, and non-signalled, controlled via
 * the signal() and reset() methods.  When in the signalled state, calls to
 * wait() will return immediately.  When in the non-signalled state,
 * calls to wait() will block until such time as the event enters the 
 * signalled state (unless a wait timeout is provided and elapses).
 **/
class CHMmanualResetEvent
{
public:
   /** Default constructor.
    * Starts in the non-signalled state.
    **/
   CHMmanualResetEvent();

   /** Destructor.
    * Non-virtual: do not inherit from this class.
    **/
   ~CHMmanualResetEvent();

   /** Set the event to the non-signalled state. */
   void reset();

   /** Set the event to the signalled state. */
   void signal();

   /** Wait for the event to enter the signalled state. */
   void wait();

   /** Wait for the event to enter the signalled state, or for the given timeout period to elapse.
    * Returns true if the event entered the signalled state, and false if the timeout period elapsed.
    **/
   CHMboolean wait( CHMuint32 TimeoutInMilliseconds );

private:
   CHMeventHandle EventHandle;
   CHMmanualResetEvent& operator=(const CHMmanualResetEvent& Orig); // not allowed 
   CHMmanualResetEvent(const CHMmanualResetEvent& Orig); // not allowed
};

#endif // end of defensive include
