#ifndef __CHM_MESSAGE_CLASS_H__
#define __CHM_MESSAGE_CLASS_H__
//---------------------------------------------------------------------------
// Copyright (C) 1997-2007 iNTERFACEWARE Inc.  All Rights Reserved
//
// Module: CHMmessageClass
//
// Author: Greg Norman
//
//---------------------------------------------------------------------------
#include <CHM/CHMtableClass.h>

class CHMmessagePrivate;
class CHMtable;
class CHMengine;

/** Message class.
 *  The message class holds the tables defined for
 *  a given message.  The message is the 'root' table
 *  which gives methods for accessing child tables that
 *  hold the message data from a parsed message.
 */
class CHMmessage 
: public CHMtable
{
public:
   CHMmessage(const CHMengineHandle& EngineHandle, 
              CHMtableHandle TableHandle, 
              size_t MessageIndex);

   /** Construct an empty message at index \c MessageIndex.  The message index
    *  is the Nth message defined in the VMD file. */
   CHMmessage(const CHMengine& Engine, size_t MessageIndex);

   /** Destructor. */
   virtual ~CHMmessage();   

   /** Get the flat-wire representation of \c this message. */
   const CHMchar* GenerateFlatWire() const;

protected:
   CHMengineHandle EngineHandle() const;
   
private:
   void AddRow();   // not required - done in the constructor
   CHMmessagePrivate* pMember;

   CHMmessage(const CHMmessage& Original);  // not allowed
   CHMmessage& operator=(const CHMmessage& Original); // not allowed
};

#endif  // end of defensive include
