#ifndef __CHM_MFC_SOCKET_TRANSPORT_H__
#define __CHM_MFC_SOCKET_TRANSPORT_H__
//---------------------------------------------------------------------------
// Copyright (C) 1997-2007 iNTERFACEWARE Inc.  All Rights Reserved
//
// Module: CHMmfcSocketTransport
//
// Author: Greg Norman
//
//---------------------------------------------------------------------------
#include <CHM/CHMtransport.h>

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
#include <afxsock.h>


// private member class
class CHMmfcSocketTransportPrivate;

/** MFC socket implementation of CHMtransport.
 * This implementation of CHMtransport inherits from the MFC
 * CSocket class to provide a TCP/IP transport layer.
 * See the on-line help for CSocket in your MFC guide for documentation.
 *
 * \deprecated
 */
class CHMmfcSocketTransport 
 : public CHMtransport, 
   public CSocket
{
public:
   /** Constructor.
    * After construction it is still necessary to call the Create
    * function.  See the on-line help for CSocket in MSDN.
    */
   CHMmfcSocketTransport();
   /** Destructor. */
   virtual ~CHMmfcSocketTransport();

   /** Close the connection. */
   virtual void Close();
   
   /** Connect to \c Host on port \c PortNumber. */
   virtual BOOL Connect(LPCTSTR Host, UINT PortNumber);
   

   enum
   {
      SOCKET_DOES_NOT_EXIST = 0x8001,
      SOCKET_NOT_A_SERVER   = 0x8002
   };

   // ClassWizard generated virtual function overrides
   //{{AFX_VIRTUAL(CHMsocket)
   /** Error on receive. */
   virtual void OnReceive(int nErrorCode);
   /** Error on accept. */
   virtual void OnAccept(int nErrorCode);
   /** Error on close. */
   virtual void OnClose(int nErrorCode);
   //}}AFX_VIRTUAL

   // Generated message map functions
   //{{AFX_MSG(CHMsocket)
   // NOTE - the ClassWizard will add and remove
   // member functions here.
   //}}AFX_MSG

   /** Send a message.
    * This method is used by the CHMwrapper class to initiate 
    * a client connection to a server by sending it a message.
    */
   virtual void SendMessage(const CHMstring& Data);
   
   /** Send a response message.
    * This method is called by the CHMwrapper class to
    * send the response back to the counter party which
    * is indexed using \c TransportIndex.
    */
   virtual void SendResponseMessage(const CHMstring& MessageData,
                                    size_t TransportIndex);   

   /** Get the transport index of this transport object.
    * For the main server socket this method will return 0.
    * For the child sockets it will be 1, 2, 3, etc.
    */
   const size_t& TransportIndex() const;

private:
   CHMmfcSocketTransportPrivate* pMember;
   // not allowed
   CHMmfcSocketTransport(const CHMmfcSocketTransport& Orig);
   CHMmfcSocketTransport& operator=(const CHMmfcSocketTransport& Orig);
   void GetDataWithoutNullChars(CHMstring& Buffer, size_t CountOfChar);
};

#endif // end of defensive include
