#ifndef __CHM_WRAPPER_H__
#define __CHM_WRAPPER_H__
//---------------------------------------------------------------------------
// Copyright (C) 1997-2007 iNTERFACEWARE Inc.  All Rights Reserved
//
// Module: CHMwrapper
//
// Author: Greg Norman
//
//---------------------------------------------------------------------------

#include <CHM/CHMminimumInclude.h>
class CHMstring;
class CHMtransport;
class CHMengine;

class CHMwrapperPrivate;

/** Class which abstracts the message wrapping protocol.
 * This is a pure virtual class designed
 * to make it easy to replace the wrapping
 * mechanisms used to delimit messages - it
 * stores the incoming message streams in
 * a vector of buffers.  Whenever a message
 * is detected by the virtual method \c DetectMessage
 * the message chunk should be extracted and
 * the \c OnMessage method called.  By default this method
 * will pass the message to the CHMengine object
 * that CHMwrapper has a pointer to.
 * 
 * The CHMengine object should then parse the
 * message and possibly send a reply using
 * its \c Reply method.  By default this reply method
 * will call the \c SendResponseMessage of the CHMwrapper
 * object which pipes it back to the CHMtransport
 * interface, etc.
 *
 * \deprecated New applications should use CHMllpServer and CHMllpClient.
 */
class CHMwrapper
{
public:
   /** Constructor. */
   CHMwrapper();

   /** Destructor. */
   virtual ~CHMwrapper();
 
   /** Wrap an outgoing message. */
   virtual void AddWrapper(CHMstring& MessageData)=0;
   
   /** Clear the buffer of the client at index \c TransportIndex. */
   void ClearData(size_t TransportIndex);

     /** Detect, and unwrap a message in an incoming stream. 
    * This routine removes the read data from \c Buffer and
    * sets the \c Message string to be equal to the message data.
    * It returns true if a message is present in the buffer; otherwise,
    * it returns false and leaves \c Buffer unaltered
    * and the content of \c Message is undefined.*/
   virtual CHMboolean DetectMessage(CHMstring& Buffer, 
                                    CHMstring& Message)=0;
                                    
   /** Get the CHMengine object assigned to this class. */
   CHMengine* Engine();
   
   /** This method calls CHMengine to parse \c MessageData. */
   virtual void OnMessage(const CHMstring& MessageData,
                          size_t           TransportIndex);

   /** Receive data from the client at index \c TransportIndex.
    * Returns the number of messages detected.
    */
   virtual size_t ReceiveData(const   CHMstring& Data,
                              size_t  TransportIndex);

   /** Send a response back to the sender.
    * Message is routed back via the \c TransportIndex argument.
    */
   void SendResponseMessage(const CHMstring& MessageData,
                            size_t           TransportIndex);

   /** Assign a CHMengine object to this class. */
   void SetEngine(CHMengine* pEngine);

   /** Get the CHMtransport object assigned to the CHMengine associated with this class. */
   CHMtransport* Transport();

private:
   CHMwrapper(CHMwrapper& Orig);
   CHMwrapper& operator=(CHMwrapper& Orig);

   CHMwrapperPrivate* pMember;
};

#endif // end of defensive include
